@@ -1302,6 +1302,64 @@ def pelvisPoseISB(mkrs, s='R'):
     return R, Opel
 
 
+def femurPoseISB(mkrs, s='R'):
+    """Calculate roto-translation matrix from femur (ISB conventions) to
+    laboratory reference frame.
+
+    Parameters
+    ----------
+    mkrs : dict
+        Markers data. Keys are marker names, values are np.ndarray N x 3,
+        where N is the number of time frames. Used names are:
+
+        - 'LE': lateral epicondyle
+        - 'ME': medial epicondyle
+        - 'HJC': hip joint center
+
+    s : {'R', 'L'}
+        Anatomical side.
+
+    Returns
+    -------
+    R : np.ndarray
+        N x 3 x 3 rotation matrix.
+
+    T : np.ndarray
+        N x 3 translation vector.
+
+    References
+    ----------
+    Wu G, Siegler S, Allard P, Kirtley C, Leardini A, Rosenbaum D, Whittle M,
+    D'Lima DD, Cristofolini L, Witte H, Schmid O, Stokes I; Standardization and
+    Terminology Committee of the International Society of Biomechanics. ISB
+    recommendation on definitions of joint coordinate system of various joints for
+    the reporting of human joint motion--part I: ankle, hip, and spine. International
+    Society of Biomechanics. J Biomech. 2002 Apr;35(4):543-8. PubMed PMID: 11934426.
+
+    """
+
+    # Define markers to use
+    LE = mkrs['LE']
+    ME = mkrs['ME']
+    HJC = mkrs['HJC']
+
+    # Create versors
+    Ofem = HJC.copy()
+    Yfem = getVersor(HJC - 0.5 * (ME + LE))
+    if s == 'R':
+        Xfem = getVersor(np.cross(Yfem, LE - ME))
+    else:
+        Xfem = getVersor(np.cross(Yfem, ME - LE))
+    Zfem = getVersor(np.cross(Xfem, Yfem))
+
+    # Create rotation matrix from shank reference frame to laboratory reference frame
+    R = np.array((Xfem.T, Yfem.T, Zfem.T))   # 3 x 3 x N
+    R = np.transpose(R, (2,1,0))  # N x 3 x 3
+
+    # Return data
+    return R, Ofem
+
+
 def pelvisPoseNoOneASI(mkrs, s='R'):
     """Calculate roto-translation matrix from pelvis (ISB conventions) to
     laboratory reference frame.
@@ -2036,14 +2094,20 @@ def getJointAngles(R1, R2, R2anglesFun=R2zxy, funInput='jointR', **kwargs):
         Rjv = np.squeeze(np.reshape(Rj,(N,9)))
         if len(Rjv.shape) == 1:
             Rjv = Rjv[None,:]
-        angles = np.apply_along_axis(R2anglesFun, 1, Rjv, **kwargs)
+        if kwargs:
+            angles = np.apply_along_axis(R2anglesFun, 1, Rjv, **kwargs)
+        else:
+            angles = np.apply_along_axis(R2anglesFun, 1, Rjv)
     elif funInput == 'segmentsR':
         R1v = np.squeeze(np.reshape(inv2(R1),(N,9)))
         R2v = np.squeeze(np.reshape(inv2(R2),(N,9)))
         Rvect = np.hstack((R1v,R2v))
         if len(Rvect.shape) == 1:
             Rvect = Rvect[None,:]
-        angles = np.apply_along_axis(R2anglesFun, 1, Rvect, **kwargs)
+        if kwargs:
+            angles = np.apply_along_axis(R2anglesFun, 1, Rvect, **kwargs)
+        else:
+            angles = np.apply_along_axis(R2anglesFun, 1, Rvect)
     elif funInput == 'jointRMatrix':
         Rj = dot3(inv2(R1), R2)
         for i in range(N):
