@@ -31,6 +31,8 @@ def performSpineAnalysis1(
                         frames = None,
                         sagSpineSplineOrder=3,
                         froSpineSplineOrder=3,
+                        useCustomSpineModelDef=False,
+                        customSpineModelDef=None,
                         savePlots=True,
                         ):
 
@@ -105,15 +107,55 @@ def performSpineAnalysis1(
         #spineDataSag = spineData[i,0:2,:].T # Np x 2
         spineDataSag = spineData[i,1::-1,:].T # Np x 2
         spineDataSagSort = np.argsort(spineDataSag[:,0])[::-1]
+        spineDataSagSort = range(spineDataSag.shape[0])
         spineDataSag = spineDataSag[spineDataSagSort,:]
+        spinePointNamesNewSag = [spinePointNamesNew[p] for p in spineDataSagSort]
         extraDataSag = extraData[i,1::-1,:].T # Ne x 2
         Np = spineDataSag.shape[0]
-        #spineLineSag = spine.create2DSpline(spineDataSag, order=sagSpineSplineOrder)
-        spineLineSag = spine.create2DPolynomial(spineDataSag, order=sagSpineSplineOrder)
         
-        # Calculate slope of spine normal at the wanted points
-        #spineLineSagDer = spine.calcSplineTangentSlopes(spineDataSag, u='only_pts')
-        spineLineSagDer = spine.calcPolynomialTangentSlopes(spineDataSag, u='only_pts', k=sagSpineSplineOrder)
+        if not useCustomSpineModelDef:
+            
+            print('using ordinary spine model ')
+            
+            #spineLineSag = spine.create2DSpline(spineDataSag, order=sagSpineSplineOrder)
+            spineLineSag = spine.create2DPolynomial(spineDataSag, order=sagSpineSplineOrder)
+            
+            # Calculate slope of spine normal at the wanted points
+            #spineLineSagDer = spine.calcSplineTangentSlopes(spineDataSag, u='only_pts')
+            spineLineSagDer = spine.calcPolynomialTangentSlopes(spineDataSag, u='only_pts', k=sagSpineSplineOrder)
+        
+        else:
+            
+            print('using custom spine model ')
+            
+            spineLineSag = np.zeros((1,2))
+            spineLineSagDer = np.zeros((1,2))
+            
+            for defPW in customSpineModelDef['model_sag']:
+                
+                print('using segment "%s"' % defPW['name'])
+                
+                pointStartPW = defPW['point_start']
+                pointStopPW = defPW['point_stop']
+                orderPW = defPW['order']
+                autoFlipXYPW = defPW['autoflip_xy']
+                sPW = spinePointNamesNewSag.index(pointStartPW)
+                ePW = spinePointNamesNewSag.index(pointStopPW) + 1
+                nPW = ePW - sPW
+                weightsPW = np.ones((nPW,))
+                if 'weights' in defPW:
+                    for weightPW in defPW['weights']:
+                        iPW = spinePointNamesNewSag.index(weightPW['point']) - sPW
+                        weightsPW[iPW] = weightPW['value']
+                
+                spineDataSagPW = spineDataSag[sPW:ePW,:]
+                
+                spineLineSagPW = spine.create2DPolynomial(spineDataSagPW, order=orderPW, weights=weightsPW, autoFlipXY=autoFlipXYPW)
+                spineLineSagDerPW = spine.calcPolynomialTangentSlopes(spineDataSagPW, u='only_pts', k=orderPW, weights=weightsPW, autoFlipXY=autoFlipXYPW)
+                
+                spineLineSag = np.vstack((spineLineSag[:-1,:], spineLineSagPW))
+                spineLineSagDer = np.vstack((spineLineSagDer[:-1,:], spineLineSagDerPW))
+        
         normalSlopesSag = -spineLineSagDer[:,1] / spineLineSagDer[:,0]
         normalInterceptsSag = spineDataSag[:,0] - normalSlopesSag * spineDataSag[:,1]
         
@@ -177,14 +219,54 @@ def performSpineAnalysis1(
 #        spineDataFro = spineData[i,2:0:-1,:].T # Np x 2
         spineDataFro = spineData[i,1:,:].T # Np x 2
         spineDataFroSort = np.argsort(spineDataFro[:,0])[::-1]
+        spineDataFroSort = range(spineDataFro.shape[0])
         spineDataFro = spineDataFro[spineDataFroSort,:]
+        spinePointNamesNewFro = [spinePointNamesNew[p] for p in spineDataFroSort]
         Np = spineDataFro.shape[0]
-#        spineLineFro = spine.create2DSpline(spineDataFro, order=froSpineSplineOrder)
-        spineLineFro = spine.create2DPolynomial(spineDataFro, order=froSpineSplineOrder)
         
-        # Calculate slope of spine normal at the wanted points
-#        spineLineFroDer = spine.calcSplineTangentSlopes(spineDataFro, u='only_pts')
-        spineLineFroDer = spine.calcPolynomialTangentSlopes(spineDataFro, u='only_pts', k=froSpineSplineOrder)
+        if not useCustomSpineModelDef:
+            
+            print('using ordinary spine model ')
+        
+    #        spineLineFro = spine.create2DSpline(spineDataFro, order=froSpineSplineOrder)
+            spineLineFro = spine.create2DPolynomial(spineDataFro, order=froSpineSplineOrder)
+            
+            # Calculate slope of spine normal at the wanted points
+    #        spineLineFroDer = spine.calcSplineTangentSlopes(spineDataFro, u='only_pts')
+            spineLineFroDer = spine.calcPolynomialTangentSlopes(spineDataFro, u='only_pts', k=froSpineSplineOrder)
+            
+        else:
+            
+            print('using custom spine model ')
+            
+            spineLineFro = np.zeros((1,2))
+            spineLineFroDer = np.zeros((1,2))
+            
+            for defPW in customSpineModelDef['model_fro']:
+                
+                print('using segment "%s"' % defPW['name'])
+                
+                pointStartPW = defPW['point_start']
+                pointStopPW = defPW['point_stop']
+                orderPW = defPW['order']
+                autoFlipXYPW = defPW['autoflip_xy']
+                sPW = spinePointNamesNewFro.index(pointStartPW)
+                ePW = spinePointNamesNewFro.index(pointStopPW) + 1
+                nPW = ePW - sPW
+                weightsPW = np.ones((nPW,))
+                if 'weights' in defPW:
+                    for weightPW in defPW['weights']:
+                        iPW = spinePointNamesNewFro.index(weightPW['point']) - sPW
+                        weightsPW[iPW] = weightPW['value']
+                
+                spineDataFroPW = spineDataFro[sPW:ePW,:]
+                
+                spineLineFroPW = spine.create2DPolynomial(spineDataFroPW, order=orderPW, weights=weightsPW, autoFlipXY=autoFlipXYPW)
+                spineLineFroDerPW = spine.calcPolynomialTangentSlopes(spineDataFroPW, u='only_pts', k=orderPW, weights=weightsPW, autoFlipXY=autoFlipXYPW)
+                
+                spineLineFro = np.vstack((spineLineFro[:-1,:], spineLineFroPW))
+                spineLineFroDer = np.vstack((spineLineFroDer[:-1,:], spineLineFroDerPW))            
+            
         normalSlopesFro = -spineLineFroDer[:,1] / spineLineFroDer[:,0]
         normalInterceptsFro = spineDataFro[:,0] - normalSlopesSag * spineDataFro[:,1]
 
