@@ -258,15 +258,22 @@ def createParamSpline(pts, k=3, retU=False):
         return spline
         
         
-def createPolynomial(pts, k=3, retU=False):
+def createPolynomial(pts, k=3, retU=False, weights=None, autoFlipXY=False):
+    # can also add: https://docs.scipy.org/doc/scipy/reference/generated/scipy.odr.polynomial.html#scipy.odr.polynomial
     x, y = pts[:,0], pts[:,1]
+    flipped = False
+    if autoFlipXY:
+#        if np.argmax(x) != 0 or np.argmin(x) != (pts.shape[0]-1):
+        if np.argmax(x) != 0:
+            x, y = y, x
+            flipped = True
     try:
-        z = np.polyfit(x, y, k)
+        z = np.polyfit(x, y, k, w=weights)
         tck, u = interpolate.splprep([x.tolist(),x.tolist()], s=0, k=1)
-        poly = z, x
+        poly = z, x, flipped
     except:
         z = np.nan * np.ones((k+1,))
-        poly = z, x
+        poly = z, x, flipped
         u = np.nan * np.ones((x.shape[0],))
     if retU:
         return poly, u
@@ -297,12 +304,14 @@ def evalSpline(spline, u):
     
 
 def evalPolynomial(poly, u):
-    z, x = poly
+    z, x, flipped = poly
     f = np.poly1d(z)
     tck, dummy = interpolate.splprep([x.tolist(),x.tolist()], s=0, k=1)
     xU = np.array(interpolate.splev(u, tck)[1])
     out = f(xU)
     p = np.array([xU, out]).T
+    if flipped:
+        p = p[:,::-1]
     return p
     
     
@@ -314,13 +323,15 @@ def evalSplineDerivative(spline, u, der=1):
     
     
 def evalPolynomialDerivative(poly, u, der=1):
-    z, x = poly
+    z, x, flipped = poly
     f = np.poly1d(z)
     f2 = np.polyder(f, m=der)
     tck, dummy = interpolate.splprep([x.tolist(),x.tolist()], s=0, k=1)
     xU = np.array(interpolate.splev(u, tck)[1])
     out = f2(xU)
     p = np.array([np.ones((xU.shape[0],)), out]).T
+    if flipped:
+        p = p[:,::-1]
     return p
     
     
